// --------------------------------------------------------------------------
// File: examples/src/LPex1.cs
// Version 10.0  
// --------------------------------------------------------------------------
//  Copyright (C) 2001-2006 by ILOG.
//  All Rights Reserved.
//  Permission is expressly granted to use this example in the
//  course of developing applications that use ILOG products.
// --------------------------------------------------------------------------
//
// LPex1.cs - Entering and optimizing an LP problem
// 
// Demonstrates different methods for creating a problem.  The user has to
// choose the method on the command line:
// 
//    LPex1  -r     generates the problem by adding constraints
//    LPex1  -c     generates the problem by adding variables
//    LPex1  -n     generates the problem by adding expressions
// 

using ILOG.Concert;
using ILOG.CPLEX;

using System.Runtime.InteropServices;
using System.Text;
using System;


public class LPex1 {

	[DllImport("cplex100.dll")]
	extern static IntPtr CPXopenCPLEX(ref int status);

	[DllImport("cplex100.dll")]
	extern static int CPXcloseCPLEX(ref IntPtr env);

	[DllImport("cplex100.dll")]
	extern static IntPtr CPXgeterrorstring (IntPtr env, int errcode, StringBuilder buffer);

	[DllImport("cplex100.dll")]
	extern static IntPtr CPXcreateprob (IntPtr env, ref int status, String name );

	[DllImport("cplex100.dll")]
	extern static int CPXfreeprob(IntPtr env, ref IntPtr lp);

	[DllImport("cplex100.dll")]
	extern static int CPXwriteprob(IntPtr env, IntPtr lp, String filename, String filetype);

	[DllImport("cplex100.dll")]
	extern static int CPXsetintparam(IntPtr env, int whichparam, int newvalue);

	[DllImport("cplex100.dll")]
	extern static void CPXchgobjsen (IntPtr env, IntPtr lp, int maxormin);

	[DllImport("cplex100.dll")]
	extern static int CPXnewcols (IntPtr env, IntPtr lp, int ccnt, double[] obj, double[] lb, double[] ub, char[] xctype, String[] colname);

	[DllImport("cplex100.dll")]
	extern static int CPXnewrows (IntPtr env, IntPtr lp, int rcnt, double[] rhs, char[] sense, double[] rngval, String[] rowname);

	[DllImport("cplex100.dll")]
	extern static int CPXaddrows (IntPtr env, IntPtr lp, int ccnt, int rcnt, int nzcnt, double[] rhs, char[] sense, int[] rmatbeg, int[] rmatind, double[] rmatval, String[] colname, String[] rowname);

	[DllImport("cplex100.dll")]
	extern static int CPXaddcols (IntPtr env, IntPtr lp, int ccnt, int nzcnt, double[] obj, int[] cmatbeg, int[] cmatind, double[] cmatval, double[] lb, double[] ub, String[] colname);

	[DllImport("cplex100.dll")]
	extern static int CPXchgcoeflist(IntPtr env, IntPtr lp, int numcoefs, int[] rowlist, int[] collist, double[] vallist);

	[DllImport("cplex100.dll")]
	extern static int CPXlpopt(IntPtr env, IntPtr lp);

	[DllImport("cplex100.dll")]
	extern static int CPXgetnumrows(IntPtr env, IntPtr lp);

	[DllImport("cplex100.dll")]
	extern static int CPXgetnumcols(IntPtr env, IntPtr lp);

	[DllImport("cplex100.dll")]
	extern static int CPXsolution(IntPtr env, IntPtr lp, ref int lpstat, ref double objval, [In, Out] double[] x, [In, Out] double[] pi, [In, Out] double[] slack, [In, Out] double[] dj);

	[DllImport("cplex100.dll")]
	extern static int CPXgetx(IntPtr env, IntPtr lp, [In, Out] double[] x, int begin, int end);

	const double CPX_INFBOUND = 1.0E+20;
	const int CPX_PARAM_SCRIND = 1035;
    const int CPX_PARAM_DATACHECK = 1056;
    const int CPX_ON = 1;
	const int CPX_MAX = -1;

   internal static void Usage() 
   {
      System.Console.Error.WriteLine("usage:   LPex1 <option>");
      System.Console.Error.WriteLine("options:       -r   build model row by row");
      System.Console.Error.WriteLine("options:       -c   build model column by column");
      System.Console.Error.WriteLine("options:       -n   build model nonzero by nonzero");
   }

   public static void Main(string[] args) {


	   /* Declare and allocate space for the variables and arrays where we
		  will store the optimization results including the status, objective
		  value, variable values, dual values, row slacks and variable
		  reduced costs. */

	   int status = -1;
	   IntPtr env = IntPtr.Zero;
	   IntPtr lp = IntPtr.Zero;


   /* Check the command line arguments */

	   if ( args.Length != 1 || args[0].ToCharArray()[0] != '-' ) 
	   {
		   Usage();
		   return;
	   }

   /* Initialize the CPLEX environment */

	   env = CPXopenCPLEX(ref status);
	   
	   if (env == IntPtr.Zero) 
	   {

		   StringBuilder errmsg = new StringBuilder(1024);
		   System.Console.Error.WriteLine("Could not open CPLEX environment.");
		   CPXgeterrorstring (env, status, errmsg);
		   System.Console.Error.WriteLine ( errmsg );
		   goto TERMINATE;
	   }


	   /* Turn on output to the screen */

	   status = CPXsetintparam (env, CPX_PARAM_SCRIND, CPX_ON);
	   if ( status != 0 ) 
	   {
		   System.Console.Error.WriteLine(
                "Failure to turn on screen indicator, error {0}.", status);
		   goto TERMINATE;
	   }

	   /* Turn on data checking */

	   status = CPXsetintparam (env, CPX_PARAM_DATACHECK, CPX_ON);
	   if ( status != 0 ) 
	   {
		   System.Console.Error.WriteLine( 
			   "Failure to turn on data checking, error {0}.", status);
		   goto TERMINATE;
	   }


	   /* Create the problem. */

	   lp = CPXcreateprob (env, ref status, "lpex1");

	   /* A returned pointer of NULL may mean that not enough memory
		  was available or there was some other problem.  In the case of 
		  failure, an error message will have been written to the error 
		  channel from inside CPLEX.  In this example, the setting of
		  the parameter CPX_PARAM_SCRIND causes the error message to
		  appear on stdout.  */


	   if ( lp == IntPtr.Zero ) 
	   {
		   System.Console.Error.WriteLine ("Failed to create LP.");
		   goto TERMINATE;
	   }

	   /* Now populate the problem with the data.  For building large
		  problems, consider setting the row, column and nonzero growth
		  parameters before performing this task. */

	   switch (args[0].ToCharArray()[1]) 
	   {
		   case 'r':
			   status = populatebyrow (env, lp);
			   break;
		   case 'c':
			   status = populatebycolumn (env, lp);
			   break;
		   case 'n':
			   status = populatebynonzero (env, lp);
			   break;
		   default:  Usage();
			   goto TERMINATE;
	   }

	   if ( status != 0 ) 
	   {
		   System.Console.Error.WriteLine("Failed to populate problem.");
		   goto TERMINATE;
	   }

	   /* Optimize the problem and obtain solution. */

	   status = CPXlpopt (env, lp);
	   if ( status != 0 ) 
	   {
		   System.Console.Error.WriteLine("Failed to optimize LP.");
		   goto TERMINATE;
	   }

	   /* The size of the problem should be obtained by asking CPLEX what
		  the actual size is, rather than using sizes from when the problem
		  was built.  cur_numrows and cur_numcols store the current number 
		  of rows and columns, respectively.  */

	   int cur_numrows = CPXgetnumrows (env, lp);
	   int cur_numcols = CPXgetnumcols (env, lp);

	   double[] x = new double[cur_numcols];
	   double[] slack = new double[cur_numrows];
	   double[] dj = new double[cur_numcols];
	   double[] pi = new double[cur_numrows];

	   int solstat = 0;
       double objval = 0.0;

	   status = CPXsolution (env, lp, ref solstat, ref objval, x, pi, slack, dj);

	   if ( status != 0 ) 
	   {
		   System.Console.Error.WriteLine("Failed to obtain solution.");
		   goto TERMINATE;
	   }

	   /* Write the output to the screen. */

	   System.Console.WriteLine ("\nSolution status = {0}", solstat);
	   System.Console.WriteLine ("Solution value  = {0,10:f6}\n", objval);

	   for (int i = 0; i < cur_numrows; i++) 
	   {
		   System.Console.WriteLine ("Row {0}:  Slack = {1,10:f6}  Pi = {2,10:f6}", i, slack[i], pi[i]);
	   }

	   for (int j = 0; j < cur_numcols; j++) 
	   {
		   System.Console.WriteLine ("Column {0}:  Value = {1,10:f6}  Reduced cost = {2,10:f6}",
			   j, x[j], dj[j]);
	   }


	   /* Finally, write a copy of the problem to a file. */

	   status = CPXwriteprob (env, lp, "lpex15.lp", null);
	   if ( status != 0 ) 
	   {
		   System.Console.Error.WriteLine("Failed to write LP to disk.");
		   goto TERMINATE;
	   }



TERMINATE:

   /* Free up the problem as allocated by CPXcreateprob, if necessary */

	if ( lp != IntPtr.Zero ) 
	{
		status = CPXfreeprob (env, ref lp);
		if ( status != 0 ) 
		{
			System.Console.Error.WriteLine("CPXfreeprob failed, error code {0}.", status);
		}
	}
   /* Free up the CPLEX environment, if necessary */

   if ( env != IntPtr.Zero ) {
      status = CPXcloseCPLEX (ref env);

      if ( status != 0 ) {

		  StringBuilder errmsg = new StringBuilder(1024);
		  System.Console.Error.WriteLine("Could not close CPLEX environment.");
		  CPXgeterrorstring (env, status, errmsg);
		  System.Console.Error.WriteLine ( errmsg );

      }
   }
     

   }


  
	/* These functions all populate the problem with data for the following
	   linear program:

		  Maximize
		   obj: x1 + 2 x2 + 3 x3
		  Subject To
		   c1: - x1 + x2 + x3 <= 20
		   c2: x1 - 3 x2 + x3 <= 30
		  Bounds
		   0 <= x1 <= 40
		  End
	 */

const int NUMROWS = 2;
const int NUMCOLS = 3;
const int NUMNZ = 6;


	/* To populate by row, we first create the columns, and then add the
	   rows.  */

	static int
		populatebyrow (IntPtr env, IntPtr lp)
	{
		int      status    = 0;
		double[]   obj = new double[NUMCOLS];
		double[]   lb = new double[NUMCOLS];
		double[]   ub = new double[NUMCOLS];
		String[]     colname = new String[NUMCOLS];
		int[]      rmatbeg = new int[NUMROWS];
		int[]      rmatind = new int[NUMNZ];
		double[]   rmatval = new double[NUMNZ];
		double[]   rhs = new double[NUMROWS];
		char[]     sense = new char[NUMROWS];
		String[]     rowname = new String[NUMROWS];

		CPXchgobjsen (env, lp, CPX_MAX);  /* Problem is maximization */

		/* Now create the new columns.  First, populate the arrays. */

		obj[0] = 1.0;      obj[1] = 2.0;          obj[2] = 3.0;

		lb[0] = 0.0;       lb[1] = 0.0;           lb[2] = 0.0;
		ub[0] = 40.0;      ub[1] = CPX_INFBOUND;  ub[2] = CPX_INFBOUND;

		colname[0] = "x1"; colname[1] = "x2";     colname[2] = "x3";

		status = CPXnewcols (env, lp, NUMCOLS, obj, lb, ub, null, colname);
		if ( status != 0 )  goto TERMINATE;

		/* Now add the constraints.  */

		rmatbeg[0] = 0;     rowname[0] = "c1";

		rmatind[0] = 0;     rmatind[1] = 1;    rmatind[2] = 2;    sense[0] = 'L';
		rmatval[0] = -1.0;  rmatval[1] = 1.0;  rmatval[2] = 1.0;  rhs[0]   = 20.0;

		rmatbeg[1] = 3;     rowname[1] = "c2";
		rmatind[3] = 0;     rmatind[4] = 1;    rmatind[5] = 2;    sense[1] = 'L';
		rmatval[3] = 1.0;   rmatval[4] = -3.0; rmatval[5] = 1.0;  rhs[1]   = 30.0;

		status = CPXaddrows (env, lp, 0, NUMROWS, NUMNZ, rhs, sense, rmatbeg,
			rmatind, rmatval, null, rowname);
		if ( status != 0 )  goto TERMINATE;

		TERMINATE:

			return (status);

	}  /* END populatebyrow */



	/* To populate by column, we first create the rows, and then add the
	   columns.  */

	static int
		populatebycolumn (IntPtr env, IntPtr lp)
	{
		int      status    = 0;
		double[]   obj = new double[NUMCOLS];
		double[]   lb = new double[NUMCOLS];
		double[]   ub = new double[NUMCOLS];
		String[]   colname = new String[NUMCOLS];
		int[]      matbeg = new int[NUMCOLS];
		int[]      matind = new int[NUMNZ];
		double[]   matval = new double[NUMNZ];
		double[]   rhs = new double[NUMROWS];
		char[]     sense = new char[NUMROWS];
		String[]   rowname = new String[NUMROWS];

		CPXchgobjsen (env, lp, CPX_MAX);  /* Problem is maximization */

		/* Now create the new rows.  First, populate the arrays. */

		rowname[0] = "c1";
		sense[0]   = 'L';
		rhs[0]     = 20.0;

		rowname[1] = "c2";
		sense[1]   = 'L';
		rhs[1]     = 30.0;

		status = CPXnewrows (env, lp, NUMROWS, rhs, sense, null, rowname);
		if ( status != 0 )   goto TERMINATE;

		/* Now add the new columns.  First, populate the arrays. */

		obj[0] = 1.0;      obj[1] = 2.0;           obj[2] = 3.0;

		matbeg[0] = 0;     matbeg[1] = 2;          matbeg[2] = 4;
      
		matind[0] = 0;     matind[2] = 0;          matind[4] = 0;
		matval[0] = -1.0;  matval[2] = 1.0;        matval[4] = 1.0;
 
		matind[1] = 1;     matind[3] = 1;          matind[5] = 1;
		matval[1] = 1.0;   matval[3] = -3.0;       matval[5] = 1.0;

		lb[0] = 0.0;       lb[1] = 0.0;            lb[2] = 0.0;
		ub[0] = 40.0;      ub[1] = CPX_INFBOUND;   ub[2] = CPX_INFBOUND;

		colname[0] = "x1"; colname[1] = "x2";      colname[2] = "x3";

		status = CPXaddcols (env, lp, NUMCOLS, NUMNZ, obj, matbeg, matind,
			matval, lb, ub, colname);
		if ( status != 0 )  goto TERMINATE;

		TERMINATE:

			return (status);

	}  /* END populatebycolumn */


	/* To populate by nonzero, we first create the rows, then create the
	   columns, and then change the nonzeros of the matrix 1 at a time.  */

	static int
		populatebynonzero (IntPtr env, IntPtr lp)
	{
		int      status    = 0;
		double[]   obj = new double[NUMCOLS];
		double[]   lb = new double[NUMCOLS];
		double[]   ub = new double[NUMCOLS];
		String[]     colname = new String[NUMCOLS];
		double[]   rhs = new double[NUMROWS];
		char[]     sense = new char[NUMROWS];
		String[]     rowname = new String[NUMROWS];
		int[]      rowlist = new int[NUMNZ];
		int[]      collist = new int[NUMNZ];
		double[]   vallist = new double[NUMNZ];

		CPXchgobjsen (env, lp, CPX_MAX);  /* Problem is maximization */

		/* Now create the new rows.  First, populate the arrays. */

		rowname[0] = "c1";
		sense[0]   = 'L';
		rhs[0]     = 20.0;

		rowname[1] = "c2";
		sense[1]   = 'L';
		rhs[1]     = 30.0;

		status = CPXnewrows (env, lp, NUMROWS, rhs, sense, null, rowname);
		if ( status !=0)   goto TERMINATE;

		/* Now add the new columns.  First, populate the arrays. */

		obj[0] = 1.0;      obj[1] = 2.0;           obj[2] = 3.0;

		lb[0] = 0.0;       lb[1] = 0.0;           lb[2]  = 0.0;
		ub[0] = 40.0;      ub[1] = CPX_INFBOUND;  ub[2]  = CPX_INFBOUND;

		colname[0] = "x1"; colname[1] = "x2";      colname[2] = "x3";

		status = CPXnewcols (env, lp, NUMCOLS, obj, lb, ub, null, colname);
		if ( status !=0)  goto TERMINATE;

		/* Now create the list of coefficients */

		rowlist[0] = 0;   collist[0] = 0;   vallist[0] = -1.0;
		rowlist[1] = 0;   collist[1] = 1;   vallist[1] = 1.0;
		rowlist[2] = 0;   collist[2] = 2;   vallist[2] = 1.0;
		rowlist[3] = 1;   collist[3] = 0;   vallist[3] = 1.0;
		rowlist[4] = 1;   collist[4] = 1;   vallist[4] = -3.0;
		rowlist[5] = 1;   collist[5] = 2;   vallist[5] = 1.0;

		status = CPXchgcoeflist (env, lp, 6, rowlist, collist, vallist);
   
		if ( status != 0 )  goto TERMINATE;

		TERMINATE:

			return (status);

	}  /* END populatebynonzero */


}
